/* dns.c of iplog v1.8 -- eric@ojnk.org */
#include "config.h"
#ifdef USE_DNS_CACHE
#include "common.h"
#include "dns.h"

#define DNS_TIMEOUT	1800
#define DNS_MAXSIZE	32
#define DNS_HASH(x)	(((x >> 24) + (x >> 16)) % (DNS_MAXSIZE + 1))

dns_entry dns_table[DNS_MAXSIZE];
int hits = 0, total = 0;

void sigusr1(int sig)
{
	(void)sig;
	if (total)
		syslog(LEVEL, "%d/%d cached; %3.1f%% success rate", hits, total, ((float)hits / (float)total) * 100);
	else
		syslog(LEVEL, "no queries have been made yet");
	signal(SIGUSR1, sigusr1);
}

void init_dns_cache(void)
{
	int n;

	signal(SIGUSR1, sigusr1);

	for (n = 0; n < DNS_MAXSIZE; n++) {
		if (dns_table[n].host != NULL)
			free(dns_table[n].host);
		dns_table[n].address = dns_table[n].when = -1;
	}
	total = hits = 0;
}

char *get_dns_cache(u_long addr)
{
	int n = DNS_HASH(addr);

	total++;

	if (dns_table[n].address == addr) {
		hits++;
		return (dns_table[n].host);
	}
	add_dns_cache(addr);

	return (dns_table[n].address == addr) ? (dns_table[n].host) : NULL;
}

void add_dns_cache(u_long addr)
{
	struct in_addr in;
	struct hostent *he;
	int n = DNS_HASH(addr);

	in.s_addr = addr;
	he = gethostbyaddr((char *)&in, sizeof(struct in_addr), AF_INET);

	dns_table[n].address = addr;
	dns_table[n].host = strdup(he ? he->h_name : inet_ntoa(in));
	time(&dns_table[n].when);
}

void expire_dns(void)
{
	time_t tm;
	int n;

	time(&tm);

	for (n = 0; n < DNS_MAXSIZE; n++)
		if (dns_table[n].address != -1UL)
			if (tm >= (dns_table[n].when + DNS_TIMEOUT)) {
				if (dns_table[n].host != NULL)
					free(dns_table[n].host);
				dns_table[n].address = dns_table[n].when = -1;
			}
}

#endif /* USE_DNS_CACHE */
