/* lockfile.h of iplog v1.8 -- eric@ojnk.org */
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/file.h>
#include "common.h"

#ifndef _PATH_VARRUN
#define	_PATH_VARRUN	"/var/run/"
#endif

char *my_getpid(void)
{
	static char pid[8];

	snprintf(pid, sizeof(pid), "%d\n", getpid());
	return (pid);
}

void initialize_lockfile(char *p)
{
	struct stat st;
	char *name, buf[8];
	int fd = 0, pid = 0;

	name = malloc(strlen(_PATH_VARRUN) + strlen(p) + 1);

	if (name == NULL)
		fatal("lockfile: malloc: %m");

	sprintf(name, "%s%s.pid", _PATH_VARRUN, p);

	if (lstat(name, &st) != -1) {
		if (S_ISREG(st.st_mode)) {
			if ((fd = open(name, O_RDONLY)) != -1) {
				if (read(fd, buf, 8) > 0)
					sscanf(buf, "%d\n", &pid);
				if (kill(pid, 0) && (errno == ESRCH)) {
					if (unlink(name) != -1)
						(void)close(fd);
					else
						fatal("couldn't remove stale lockfile %s", name);
				} else if (flock(fd, LOCK_EX | LOCK_NB) == -1)
					fatal("%s is already running (pid: %d)", p, pid);
				else
					(void)close(fd);
			}
		} else
			fatal("lockfile %s isn't a regular file", name);
	}
	if ((fd = open(name, O_RDWR | O_CREAT | O_TRUNC, 0644)) != -1) {
		write(fd, my_getpid(), strlen(my_getpid()));

		if (flock(fd, LOCK_EX | LOCK_NB) == -1)
			fatal("lockfile not initialized (%m)");
	}
	free(name);
	return;
}
