/* portscan.c of iplog v1.8 -- eric@ojnk.org */
#include "config.h"
#ifdef USE_PORTSCAN_DETECTION
#include "common.h"

portscan_entry scan_table[PORTSCAN_MAXSIZE];
time_t lastexpire = 0;

int portscan(struct in_addr *in)
{
	int n = PORTSCAN_HASH(in->s_addr);
	time_t current;

	time(&current);

	if (scan_table[n].addr == in->s_addr) {
		if (scan_table[n].bad) {
			if (current >= (scan_table[n].last + PORTSCAN_WAIT)) {
				syslog(LEVEL, "No longer in scan mode for %s", inet_ntoa(*in));
				scan_table[n].bad = 0;
				scan_table[n].count = 1;
				scan_table[n].last = current;
				return 0;
			} else {
				scan_table[n].last = current;
				return 1;
			}
		} else {
			if (scan_table[n].last != current) {
				scan_table[n].count = 1;
				scan_table[n].last = current;
				return 0;
			}
			if (scan_table[n].count++ >= PORTSCAN_THRESHOLD) {
				scan_table[n].bad = 1;
				scan_table[n].last = current;
				syslog(LEVEL, "port scanning from %s (threshold %d ports/sec met)", inet_ntoa(*in), PORTSCAN_THRESHOLD);
				return 1;
			} else {
				scan_table[n].last = current;
				return 0;
			}
		}
	} else {
		scan_table[n].addr = in->s_addr;
		scan_table[n].last = current;
		scan_table[n].count = 1;
		scan_table[n].bad = 0;
	}
	return 0;
}

void init_portscan(void)
{
	int n;

	for (n = 0; n < PORTSCAN_MAXSIZE; n++) {
		scan_table[n].addr = scan_table[n].last = -1;
		scan_table[n].bad = scan_table[n].count = 0;
	}
}

void expire_portscan(void)
{
	struct in_addr in;
	time_t tm;
	int n;

	time(&tm);

	for (n = 0; n < PORTSCAN_MAXSIZE; n++)
		if (scan_table[n].addr != -1UL)
			if (tm >= (scan_table[n].last + PORTSCAN_WAIT)) {
				if (scan_table[n].bad) {
					in.s_addr = scan_table[n].addr;
					syslog(LEVEL, "No longer in scan mode for %s", inet_ntoa(in));
				}
				scan_table[n].addr = scan_table[n].last = -1;
				scan_table[n].bad = scan_table[n].count = 0;
			}
}

#endif /* USE_PORTSCAN_DETECTION */
