/* smurf.h of iplog v1.8 -- eric@ojnk.org */
#include "config.h"
#ifdef DETECT_SMURF
#include "common.h"

float kbps = 0.0;
u_long *ips, *ignore_ips;
time_t last = 0, lasta = 0;
int count = 0, ignore_count = 0, smurf_mode = 0, pps = 0;

void sigalrm(int stat)
{
	int freed = 0;

	(void)stat;
	if (smurf_mode) {
		if (count > 0) {
			free(ips);
			freed += sizeof(u_long) * count;
		}
		if (ignore_count > 0) {
			free(ignore_ips);
			freed += sizeof(u_long) * ignore_count;
		}
		syslog(LEVEL, "No longer in smurf mode (%d bytes memory)", freed);
	}
	ignore_count = count = smurf_mode = 0;
	signal(SIGALRM, sigalrm);
}

/* based on buglord's SmurfLog, but modified heavily to work in iplog */
int smurf(struct ip *ip)
{
	time_t current;
	u_long the_ip;
	u_char *p = NULL;

	time(&current);

	if (smurf_mode) {
		p = (u_char *) & ip->ip_src.s_addr;

		p[3] = 0;

		the_ip = ip->ip_src.s_addr;

		if (count > SMURF_LIMIT)
			return 1;

		if (bsearch(&the_ip, ips, count, sizeof(u_long), comp) == NULL) {
			if ((ips = realloc(ips, sizeof(u_long) * (count + 1))) == NULL)
				fatal("realloc: out of memory");

			ips[count++] = the_ip;
			qsort(ips, count, sizeof(u_long), comp);
		} else if (bsearch(&the_ip, ignore_ips, ignore_count, sizeof(u_long), comp) == NULL) {
			syslog(LEVEL, "smurf attack: %s/24 (%d bytes)", inet_ntoa(ip->ip_src), UNFIX(ip->ip_len) - (ip->ip_p == IPPROTO_ICMP ? ICMP_SIZE : UDP_SIZE));

			if ((ignore_ips = realloc(ignore_ips, sizeof(u_long) * (ignore_count + 1))) == NULL)
				fatal("realloc: out of memory");

			ignore_ips[ignore_count++] = the_ip;
			qsort(ignore_ips, ignore_count, sizeof(u_long), comp);
		}
		if ((lasta + 5) <= time(NULL)) {
			alarm(SMURF_WAIT);
			lasta = current;
		}
		return 1;
	} else {
		kbps += UNFIX(ip->ip_len) / 1024.0;
		pps++;

		if ((current - last) >= 1 || (current < last)) {
			pps = 0;
			kbps = 0.0;
			last = current;
			return 0;
		} else if ((pps >= THRESH_PPS && kbps > THRESH_MIN) || kbps >= THRESH_KBPS) {
			smurf_mode = 1;
			syslog(LEVEL, "threshold met (%ld packets/sec, %4.2fkb/sec)", pps, kbps);
			last = current;
			return 1;
		}
		last = current;
		return 0;
	}
}

#endif /* DETECT_SMURF */
