/* tcplog.c of iplog v1.8 -- eric@ojnk.org */
#include "config.h"
#include "common.h"
#include <sys/wait.h>
#include "ident.h"
#include "tcplog.h"

#ifdef RETRIEVE_IDENTD
void sigchld(int stat)
{
	while (waitpid(-1, &stat, WNOHANG) > 0);
	signal(SIGCHLD, sigchld);
}
#endif

#if defined(USE_DNS_CACHE) || defined(USE_PORTSCAN_DETECTION)
void sighup(int sig)
{
	(void)sig;					/* make gcc shutup */
#ifdef USE_DNS_CACHE
	init_dns_cache();
#endif
#ifdef USE_PORTSCAN_DETECTION
	init_portscan();
#endif
	signal(SIGHUP, sighup);
}

void sigalrm(int sig)
{
	(void)sig;
#ifdef USE_DNS_CACHE
	expire_dns();
#endif
#ifdef USE_PORTSCAN_DETECTION
	expire_portscan();
#endif
	signal(SIGALRM, sigalrm);
	alarm(20);
}

#endif

int main(void)
{
	int sock;
	time_t tm;
	struct tcp_header hdr;

#if defined(DETECT_FIN) || defined(DETECT_NULL)
	unsigned short flags = 0;

#endif
#ifdef DETECT_FIN
	time_t finlast = 0;
	u_char fincount = 0;
	u_long fin_last = 0;

#endif
#ifdef DETECT_NULL
	time_t nulllast = 0;
	u_long null_last = 0;

#endif
#ifdef DETECT_BOGUS
	u_long last_bogus = 0;

#endif
#ifdef RETRIEVE_IDENTD
	char *user = NULL, *hostname = NULL;

#endif
	time_t synlast = 0;
	u_char syncount = 0;

	if ((sock = socket(AF_INET, SOCK_RAW, IPPROTO_TCP)) == -1) {
		perror("socket");
		exit(EXIT_FAILURE);
	}
	openlog("tcplog", LOGGING_OPTIONS, LOGGING_FACILITY);
#ifdef RETRIEVE_IDENTD
	signal(SIGCHLD, sigchld);

#endif
#if defined(USE_PORTSCAN_DETECTION) || defined(USE_DNS_CACHE)
	signal(SIGHUP, sighup);
#else
	signal(SIGHUP, SIG_IGN);
#endif

#ifdef USE_PORTSCAN_DETECTION
	init_portscan();
#endif
#ifdef USE_DNS_CACHE
	init_dns_cache();
	signal(SIGALRM, sigalrm);
#endif

	fork_to_background();
	initialize_lockfile("tcplog");
	drop_privileges();

#if defined(USE_PORTSCAN_DETECTION) || defined(USE_DNS_CACHE)
	alarm(20);
#endif

	for (;;) {
#if defined(USE_PORTSCAN_DETECTION) || defined(USE_DNS_CACHE)
		errno = 0;
		read(sock, &hdr, sizeof(hdr));
		if (errno == EINTR)
			continue;
#else
		read(sock, &hdr, sizeof(hdr));
#endif
		time(&tm);

		if (syncount) {
			syncount -= ((tm - synlast) >> 2);
			synlast = tm;
		}
#ifdef DETECT_BOGUS
		/* Nmap and Queso use a bogus tcp flag to "fingerprint" OS'es.. */
		if ((hdr.tcp.th_flags & TH_BOG) && (last_bogus != hdr.ip.ip_src.s_addr)) {
			last_bogus = hdr.ip.ip_src.s_addr;
			syslog(LEVEL, "bogus tcp flags set by %s (%s)", hostname_lookup(hdr.ip.ip_src, (syncount != SYN_FLOOD)), inet_ntoa(hdr.ip.ip_src));
		}
#endif

#ifdef DETECT_NULL
		if (!hdr.tcp.th_flags) {
			if (tm >= (nulllast + 60))
				flags &= ~NULL_LOGGED;
			if ((hdr.ip.ip_src.s_addr == null_last) && !(flags & NULL_LOGGED)) {
				syslog(LEVEL, "null scan detected from %s (%s)", hostname_lookup(hdr.ip.ip_src, (syncount != SYN_FLOOD)), inet_ntoa(hdr.ip.ip_src));
				flags |= NULL_LOGGED;
				nulllast = tm;
			} else if (hdr.ip.ip_src.s_addr != null_last) {
				flags &= ~NULL_LOGGED;
				null_last = hdr.ip.ip_src.s_addr;
			}
			continue;
		}
#endif

#ifdef DETECT_FIN
		if ((hdr.tcp.th_flags & TH_FIN) && !(hdr.tcp.th_flags & TH_ACK)) {
			if (tm >= (finlast + 60))
				flags &= ~FIN_LOGGED;
			if ((hdr.ip.ip_src.s_addr == fin_last) && !(flags & FIN_LOGGED)) {
				if (++fincount >= 3) {
					syslog(LEVEL, "%s scan detected from %s (%s)", (hdr.tcp.th_flags & TH_URG) ? "Xmas" : "fin", hostname_lookup(hdr.ip.ip_src, (syncount != SYN_FLOOD)), inet_ntoa(hdr.ip.ip_src));
					flags |= FIN_LOGGED;
					finlast = tm;
				}
			} else if (hdr.ip.ip_src.s_addr != fin_last) {
				flags &= ~FIN_LOGGED;
				fincount = 1;
				fin_last = hdr.ip.ip_src.s_addr;
			}
			continue;
		}
#endif
		if ((hdr.tcp.th_flags & TH_SYN) && !(hdr.tcp.th_flags & TH_ACK)) {
			synlast = tm;

			if (syncount != SYN_FLOOD)
				syncount++;

#ifdef USE_PORTSCAN_DETECTION
			if (portscan(&hdr.ip.ip_src))
				continue;
#endif

#ifndef LOG_HTTP_CONNECTS
			if (((UNFIX(hdr.tcp.th_dport) == 80) || (UNFIX(hdr.tcp.th_dport) == 8080) || (UNFIX(hdr.tcp.th_dport) == 1080)) && UNFIX(hdr.tcp.th_sport) >= IPPORT_RESERVED)
				continue;
#endif

#ifndef LOG_FTP_TRANSFERS
			if (UNFIX(hdr.tcp.th_sport) == IPPORT_FTP && UNFIX(hdr.tcp.th_dport) >= IPPORT_RESERVED)
				continue;
#endif
#ifdef RETRIEVE_IDENTD
			if (hdr.tcp.th_dport != IDENTD_PORT && (hdr.tcp.th_sport != IPPORT_FTP || UNFIX(hdr.tcp.th_dport) <= IPPORT_RESERVED) && (syncount != SYN_FLOOD) && l_port(hdr.tcp.th_dport)) {
				hostname = strdup(hostname_lookup(hdr.ip.ip_src, (syncount != SYN_FLOOD)));
				if (!fork()) {
					user = get_identd_data(&hdr);

					if (user != NULL) {
						syslog(LEVEL, "%s connection attempt from %s@%s:%u", service_lookup(hdr.tcp.th_dport, "tcp"), user, hostname, UNFIX(hdr.tcp.th_sport));
						free(user);
					} else
						syslog(LEVEL, "%s connection attempt from %s:%u", service_lookup(hdr.tcp.th_dport, "tcp"), hostname, UNFIX(hdr.tcp.th_sport));

					exit(EXIT_SUCCESS);
				}
				free(hostname);
			} else
				syslog(LEVEL, "%s connection attempt from %s:%u", service_lookup(hdr.tcp.th_dport, "tcp"), hostname_lookup(hdr.ip.ip_src, (syncount != SYN_FLOOD)), UNFIX(hdr.tcp.th_sport));
		}
#else
			syslog(LEVEL, "%s connection attempt from %s:%u", service_lookup(hdr.tcp.th_dport, "tcp"), hostname_lookup(hdr.ip.ip_src, (syncount != SYN_FLOOD)), UNFIX(hdr.tcp.th_sport));
		}
#endif
	}
	(void)close(sock);
}

static char *hostname_lookup(struct in_addr in, int dohost)
{
	if (!dohost) {
		return (inet_ntoa(in));
	} else {
#ifdef USE_DNS_CACHE
		char *host;

		if ((host = get_dns_cache(in.s_addr)) != NULL)
			return (host);
	}
#else
		struct hostent *he;
		he = gethostbyaddr((char *)&in, sizeof(struct in_addr), AF_INET);

		if (he == NULL)
			return (inet_ntoa(in));
		else
			return (he->h_name);
	}
#endif
	return ("unknown");
}
