/* udplog.c of iplog v1.8 -- eric@ojnk.org */
#include "config.h"
#include "common.h"
#include "udplog.h"

int main(void)
{
	struct udp_packet pkt;
	int sock;

#ifdef DETECT_TRACEROUTE
	time_t lasttr = 0;
	u_long last = 0;

#endif

	if ((sock = socket(AF_INET, SOCK_RAW, IPPROTO_UDP)) == -1) {
		perror("socket");
		exit(EXIT_FAILURE);
	}
	openlog("udplog", LOGGING_OPTIONS, LOGGING_FACILITY);
	signal(SIGHUP, sighup);
#ifdef DETECT_SMURF
	signal(SIGALRM, sigalrm);
#endif

	fork_to_background();
	initialize_lockfile("udplog");
	drop_privileges();
	readns();

	for (;;) {
	  next:
		errno = 0;
		read(sock, &pkt, sizeof(pkt));

		if (errno == EINTR)
			continue;

		switch (UNFIX(pkt.udp.source)) {
			case PORT_DOMAIN:
				if (bsearch(&pkt.ip.ip_src.s_addr, ns, ns_count, sizeof(u_long), comp) != NULL)
					goto next;
#ifdef IGNORE_NFS
			case PORT_NFS:
				goto next;
#endif
#ifdef DETECT_SMURF
			case IPPORT_ECHO:
				if (smurf(&pkt.ip))
					goto next;
#endif
			default:
#ifdef DETECT_TRACEROUTE
				if ((pkt.ip.ip_ttl == 1) && (UNFIX(pkt.udp.dest) > 30000)) {
					if ((last != pkt.ip.ip_src.s_addr) || (time(NULL) >= (lasttr + 15))) {
						syslog(LEVEL, "traceroute from %s (%s)", hostname_lookup(pkt.ip.ip_src), inet_ntoa(pkt.ip.ip_src));
						time(&lasttr);
						last = pkt.ip.ip_src.s_addr;
					}
					goto next;
				}
#endif
				syslog(LEVEL, "dgram to %s from %s:%u (%u bytes)", service_lookup(pkt.udp.dest, "udp"), inet_ntoa(pkt.ip.ip_src), UNFIX(pkt.udp.source), UNFIX(pkt.ip.ip_len) - UDP_SIZE);
				break;
		}
	}
	(void)close(sock);
}

#ifdef DETECT_TRACEROUTE
static char *hostname_lookup(struct in_addr in)
{
	struct hostent *he;

	he = gethostbyaddr((char *)&in, sizeof(struct in_addr), AF_INET);

	if (he != NULL)
		return (he->h_name);
	else
		return (inet_ntoa(in));
}
#endif

void sighup(int stat)
{
	(void)stat;
	readns();
	signal(SIGHUP, sighup);
}

void readns(void)
{
	FILE *fp;
	char line[BUFSIZ], host[18];

	if ((fp = fopen(_PATH_RESCONF, "r")) == NULL)
		fatal("%s: %m", _PATH_RESCONF);

	if (ns != NULL)
		free(ns);

	ns_count = 0;

	while (fgets(line, sizeof(line) - 1, fp) != NULL) {
		if (sscanf(line, "nameserver %18s", host) == 1) {
			if (inet_addr(host) == -1UL)
				continue;

			ns = realloc(ns, sizeof(u_long) * (ns_count + 1));

			if (ns == NULL)
				fatal("realloc: out of memory");

			ns[ns_count++] = inet_addr(host);
		}
	}

	if (ns_count > 0)
		qsort(ns, ns_count, sizeof(u_long), comp);

	(void)fclose(fp);

	syslog(LEVEL, "cached %d nameserver%s", ns_count, ns_count == 0 ? "" : "s");
	return;
}
